# hack-the-news

## Prerequisites
  * Python 3.5.2+
  * (Optional, but highly recommended) Create and use a virtual environment for isolating from system site directories - there are many options, but the most recommended is `venv` by the time of writing this. Creating virtual environment with `venv`: `python3 -m venv <path_to_your_virtual_env>`. Activating it is done with `source <path_to_your_virtual_env>/bin/activate`.
  * To install the required packages, activate the virtual environment and run `pip3 install -r requirements.txt` or if you are without a virtual environment, run `pip3 install --user -r requirements.txt` to install them in a Python user install directory instead of a system directory.
  * Add file ```config.json``` to root with (change the values by your preferences):
  ```
  {
    "persist_results": true,
    "results_file": "results/my_results_file.json",
    "n_jobs": 1,
    "ibm_api_key": "<YOUR KEY HERE>"
  }
  ```
The `ibm_api_key` is only needed if you want to generate the emotions features cache for a different set of articles.

## Structure

`data/`: Contains data sets for training, testing, validation and other.

`data_utils.py`: Utility functions and types for working with article sentences.

`documentation/`: Contains the article for the datathon.

`main.py`: The main configurable pipeline for training, testing and running models on data sets.

`models/`: Contains all our models which have a `fit()` and `predict()` method.

`notebooks/`: Contains Jupyter notebooks.

`predictions/`: Contains predictions.

`README.md`: Documentation of the repository.

`requirements.txt`: List of Python pip dependencies.

`results/`: Contains JSON files with evaluation results of the trained models and their parameters.

`tools/`: Contains tools, provided by the Datathon organizers.

`transformers/`: Contains all feature transformers (e.g. sentiment, subjectivity, readability, etc.).

`utils.py`: A set of utility methods, e.g `compute_score` which is used for model evaluation, and others.

## Generating Emotion Features
Emotion features are extracted from IBM Watson Natural Language Understanding API. You need to have an IBM Cloud account with a NLU project. Get the API key for it and set it in your `config.json`. After that run the following two scripts:
* `python fetch_emotions_features.py <path-to-articles-dir> <start-range> <end-range>` - Fetches the emotions for each non-empty line in the articles in the specified folder. The folder must have the expected structure for task 2, i.e. text files `article<ARTICLE_ID>.txt` and corresponding label files `article<ARTICLE_ID>.task2.labels` (possibly with `?` labels). `<start-range>` and `<end-range>` allow the script to work on a subset of the data. Extracted features will be stored as .csv files in the `transformers/emotions_cache` folder.
* `python merge_emotions_cache.py` - Goes through all .csv files in the `transformers/emotions_cache` folder and merges the results, preserving the latest data available for each sentence_id, then stores all results in `transformers/emotions_cache/all_emotions.csv`.

During training and generating predictions, the IBM API is not used - values are only read from the `all_emotions.csv` file. If there are missing sentences in the cache, errors will occur as the dimensions of features will not match.