from sklearn.linear_model import LogisticRegression
from sklearn.feature_extraction.text import TfidfVectorizer

from models.base_estimator import BaseEstimator
from utils import flatten, get_n_jobs

class LogRegBaseline(BaseEstimator):
    def __init__(self):
        self.params = {
            'description': 'Logistic Regression with TFIDF (Baseline)',
            'logreg_clf_params': {
                'penalty': 'l2',
                'C': 0.8,
                'class_weight': 'balanced',
                'solver': 'lbfgs',
                'max_iter': 3000,
            }
        }

    def fit(self, train_x, train_y):
        train_x = [x.sentence for x in train_x]

        self.vectorizer = TfidfVectorizer(sublinear_tf=True)

        tfidf_features = self.vectorizer.fit_transform(train_x)

        self.model = LogisticRegression(**self.params['logreg_clf_params'])
        self.model.fit(tfidf_features, train_y)

    def predict(self, test_x):
        test_x = [x.sentence for x in test_x]

        tfidf_features = self.vectorizer.transform(test_x)

        return self.model.predict(tfidf_features)

    def get_grid_params(self):
        return {
            'max_iter': (5, 100),
            'solver': ('liblinear', 'sag'),
        }
