from sklearn.linear_model import LogisticRegression

from models.base_estimator import BaseEstimator

from transformers import has_confusing_words

class LogRegConfusingWords(BaseEstimator):
    def __init__(self):
        self.params = {
            'description': 'Logistic Regression with confusion words',
            'logreg_clf_params': {
                'penalty': 'l2',
                'C': 0.8,
                'class_weight': 'balanced',
                'solver': 'lbfgs',
                'max_iter': 3000,
            }
        }

    def fit(self, train_x, train_y):
        train_x = [x.sentence for x in train_x]

        features = has_confusing_words(train_x)

        self.model = LogisticRegression(**self.params['logreg_clf_params'])
        self.model.fit(features, train_y)

    def predict(self, test_x):
        test_x = [x.sentence for x in test_x]

        features = has_confusing_words(test_x)

        return self.model.predict(features)

    def get_grid_params(self):
        return {
            'max_iter': (5, 100),
            'solver': ('liblinear', 'sag'),
        }
