from sklearn.linear_model import LogisticRegression
import numpy as np
import scipy
from models.base_estimator import BaseEstimator
from utils import flatten, get_n_jobs
from transformers import EmotionsExtractor, readability, merge_pos_counts, subjectivity, polarity
from sklearn.feature_extraction.text import TfidfVectorizer

class LogRegEmotions(BaseEstimator):
    def __init__(self):
        self.emotions_extractor = EmotionsExtractor()
        self.vectorizer = TfidfVectorizer()

        self.params = {
            'description': 'Logistic Regression with Emotion, readability, POS, subjectivity, tf-idf features',
            'logreg_clf_params': {
                'penalty': 'l2',
                'dual': False,
                'tol': 0.0001,
                'C': 0.8,
                'fit_intercept': True,
                'intercept_scaling': 1,
                'class_weight': 'balanced',
                'random_state': None,
                'solver': 'liblinear',
                'max_iter': 3000,
                'multi_class': 'ovr',
                'verbose': 0,
                'warm_start': False,
                'n_jobs': 1,
            }
        }

    def prepare_features(self, data_x):
        sentences = [article.sentence for article in data_x]
        # pos = merge_pos_counts(sentences)
        emotions = self.emotions_extractor.load_features(data_x)

        # tfidf = self.vectorizer.transform(sentences)
        # read = readability(sentences)
        # subj = subjectivity(sentences)
        # pol = polarity(sentences)
        # return read
        # return np.concatenate((
        #     read, emotions
        # ), axis=1)

        # return scipy.sparse.hstack([
        #     emotions, read, pos, tfidf, subj, pol
        # ])
        return emotions
        # return scipy.sparse.hstack([
        #     emotions
        # ])

    def fit(self, train_x, train_y):
        self.vectorizer.fit([article.sentence for article in train_x])

        train_x = self.prepare_features(train_x)

        self.model = LogisticRegression(**self.params['logreg_clf_params'])
        self.model.fit(train_x, train_y)

    def predict(self, test_x):
        test_x = self.prepare_features(test_x)

        return self.model.predict(test_x)

    def get_grid_params(self):
        return {
            'max_iter': (5, 100),
            'solver': ('liblinear', 'sag'),
        }
