from nltk.corpus import stopwords
from nltk.tokenize import RegexpTokenizer
from sklearn.linear_model import LogisticRegression
import numpy as np

from models.base_estimator import BaseEstimator
from transformers.exclamations import exclamation_counts, exclamations_count

def read_exclamations():
    with open('data/external/exclamations.txt', 'r', encoding='utf-8') as file:
        exclamations = [line.rstrip('\n') for line in file.readlines()]
    return exclamations

class LogRegExclamations(BaseEstimator):
    def __init__(self):
        self.params = {
            'description': 'Logistic Regression with loaded language',
            'logreg_clf_params': {
                'penalty': 'l2',
                'C': 0.8,
                'class_weight': 'balanced',
                'solver': 'lbfgs',
                'max_iter': 3000,
            }
        }
        self.tokenizer = RegexpTokenizer(r'\w+')
        self.stop_words = set(stopwords.words('english'))
        self.exclamations = read_exclamations()

    def fit(self, train_x, train_y):
        train_x = [x.sentence for x in train_x]
        train_x = [self.clean_stop_words(sentence) for sentence in train_x]

        # train_x = np.array([exclamation_counts(words, self.exclamations) for words in train_x])
        train_x = np.array([exclamations_count(words, self.exclamations) for words in train_x]).reshape(-1, 1)

        self.model = LogisticRegression(**self.params['logreg_clf_params'])
        self.model.fit(train_x, train_y)

    def predict(self, test_x):
        test_x = [x.sentence for x in test_x]
        test_x = [self.clean_stop_words(sentence) for sentence in test_x]

        # test_x = np.array([exclamation_counts(words, self.exclamations) for words in test_x])
        test_x = np.array([exclamations_count(words, self.exclamations) for words in test_x]).reshape(-1, 1)

        return self.model.predict(test_x)

    def clean_stop_words(self, sentence):
        word_tokens = self.tokenizer.tokenize(sentence)
        return [word_token for word_token in word_tokens if not word_token in self.stop_words]
