from sklearn.linear_model import LogisticRegression
import numpy as np

from models.base_estimator import BaseEstimator
from transformers.loaded_language import loaded_language_phrase_counts, loaded_language_phrases_count

def read_loaded_language_phrases():
    with open('data/external/loaded_language_phrases.txt', 'r', encoding='utf-8') as file:
        phrases = [line.rstrip('\n') for line in file.readlines()]
    return phrases


class LogRegLoadedLanguage(BaseEstimator):
    def __init__(self):
        self.params = {
            'description': 'Logistic Regression with loaded language',
            'logreg_clf_params': {
                'penalty': 'l2',
                'C': 0.8,
                'class_weight': 'balanced',
                'solver': 'lbfgs',
                'max_iter': 3000,
            }
        }
        self.phrases = read_loaded_language_phrases()

    def fit(self, train_x, train_y):
        train_x = [x.sentence for x in train_x]

        # train_x = np.array([loaded_language_phrase_counts(sentence, self.phrases) for sentence in train_x])
        train_x = np.array([loaded_language_phrases_count(sentence, self.phrases) for sentence in train_x]).reshape(-1, 1)

        self.model = LogisticRegression(**self.params['logreg_clf_params'])
        self.model.fit(train_x, train_y)

    def predict(self, test_x):
        test_x = [x.sentence for x in test_x]

        # test_x = np.array([loaded_language_phrase_counts(sentence, self.phrases) for sentence in test_x])
        test_x = np.array([loaded_language_phrases_count(sentence, self.phrases) for sentence in test_x]).reshape(-1, 1)

        return self.model.predict(test_x)
