from sklearn.linear_model import LogisticRegression

from models.base_estimator import BaseEstimator
from transformers.proper_nouns import singular_proper_noun_max_counts, plural_proper_noun_max_counts

class LogRegProperNouns(BaseEstimator):
    def __init__(self):
        self.params = {
            'description': 'Logistic Regression with loaded language',
            'logreg_clf_params': {
                'penalty': 'l2',
                'C': 0.8,
                'class_weight': 'balanced',
                'solver': 'lbfgs',
                'max_iter': 3000,
            }
        }

    def fit(self, train_x, train_y):
        train_x = [x.sentence for x in train_x]

        train_x = singular_proper_noun_max_counts(train_x)

        self.model = LogisticRegression(**self.params['logreg_clf_params'])
        self.model.fit(train_x, train_y)

    def predict(self, test_x):
        test_x = [x.sentence for x in test_x]

        test_x = singular_proper_noun_max_counts(test_x)

        return self.model.predict(test_x)
