from sklearn.linear_model import LogisticRegression

from models.base_estimator import BaseEstimator
from transformers import compute_sentiment

class LogRegSentiment(BaseEstimator):
    def __init__(self):
        self.params = {
            'description': 'Logistic Regression with sentiment features',
            'logreg_clf_params': {
                'penalty': 'l2',
                'C': 0.8,
                'class_weight': 'balanced',
                'solver': 'lbfgs',
                'max_iter': 3000,
            }
        }

    def fit(self, train_x, train_y):
        train_x = [x.sentence for x in train_x]

        lexical_features = compute_sentiment(train_x)

        self.model = LogisticRegression(**self.params['logreg_clf_params'])
        self.model.fit(lexical_features, train_y)

    def predict(self, test_x):
        test_x = [x.sentence for x in test_x]

        lexical_features = compute_sentiment(test_x)

        return self.model.predict(lexical_features)
