from sklearn.linear_model import LogisticRegression
from sklearn.feature_extraction.text import TfidfVectorizer

from models.base_estimator import BaseEstimator
from utils import flatten, get_n_jobs

import numpy as np

class LogRegSentlen(BaseEstimator):
    def __init__(self):
        self.params = {
            'description': 'Logistic Regression with sentence lengths (Baseline)',
            'logreg_clf_params': {
                'C': 0.8,
                'penalty': 'l2',
                'class_weight': 'balanced',
                'solver': 'lbfgs',
                'max_iter': 3000,
            }
        }

    def fit(self, train_x, train_y):
        train_x = [x.sentence for x in train_x]

        train_x = np.array([len(sentence) for sentence in train_x ]).reshape(-1, 1)

        self.model = LogisticRegression(**self.params['logreg_clf_params'])
        self.model.fit(train_x, train_y)

    def predict(self, test_x):
        test_x = [x.sentence for x in test_x]

        test_x = np.array([len(sentence) for sentence in test_x ]).reshape(-1, 1)

        return self.model.predict(test_x)
