import numpy as np
import transformers
from models.base_estimator import BaseEstimator
from sklearn.linear_model import LogisticRegression

class LogRegSubjectivity(BaseEstimator):
    def __init__(self):
        self.params = {
            'description': 'Logistic Regression with subjectivity and polarity',
            'logreg_clf_params': {
                'penalty': 'l2',
                'C': 0.8,
                'class_weight': 'balanced',
                'solver': 'lbfgs',
                'max_iter': 3000,
            },
            'readability_indices': ['flesch_reading_ease', 'smog_index', 'difficult_words']
        }

    def fit(self, train_x, train_y):
        train_x = [x.sentence for x in train_x]

        features = self.transform_features(train_x)

        self.model = LogisticRegression(**self.params['logreg_clf_params'])
        self.model.fit(features, train_y)

    def predict(self, test_x):
        test_x = [x.sentence for x in test_x]

        features = self.transform_features(test_x)

        return self.model.predict(features)

    def transform_features(self, data_x):
        features = np.concatenate((
            list(transformers.subjectivity(data_x)),
            list(transformers.polarity(data_x)),
            # transformers.count_adjectives(data_x),
            # transformers.count_adverbs(data_x),
            # transformers.readability(data_x, indices=self.params['readability_indices'])
        ), axis = 1)
        return features
