import numpy as np
import transformers
from models.base_estimator import BaseEstimator

import tensorflow as tf

class NNSubjectivity(BaseEstimator):
    def __init__(self):
        self.params = {
            'description': 'NN with subjectivity, polarity, readability, adj., adv.',
            'nn_clf_params': {
                'epochs': 100,
                'verbose': 0
            },
            'readability_indices': ['flesch_reading_ease', 'smog_index', 'difficult_words']
        }

    def fit(self, train_x, train_y):
        train_x = [x.sentence for x in train_x]

        features = self.transform_features(train_x)

        self.model = tf.keras.models.Sequential()
        self.model.add(tf.keras.layers.Dense(16, activation='relu'))
        self.model.add(tf.keras.layers.Dense(64, activation='relu'))
        self.model.add(tf.keras.layers.Dense(16, activation='relu'))
        self.model.add(tf.keras.layers.Dense(1, activation='sigmoid'))

        self.model.compile(loss='binary_crossentropy', optimizer='adam', metrics=['accuracy'])
        self.model.fit(features, train_y, **self.params['nn_clf_params'])

    def predict(self, test_x):
        test_x = [x.sentence for x in test_x]

        features = self.transform_features(test_x)

        return self.model.predict(features)

    def transform_features(self, data_x):
        features = np.concatenate((
            transformers.subjectivity(data_x),
            transformers.polarity(data_x),
            transformers.count_adjectives(data_x),
            transformers.count_adverbs(data_x),
            transformers.readability(data_x, indices=self.params['readability_indices'])
        ), axis = 1)
        return features

    def get_grid_params(self):
        return {

        }
