import numpy as np
from sklearn.ensemble import RandomForestClassifier
from models.base_estimator import BaseEstimator
import transformers

class RandomForest(BaseEstimator):
    def __init__(self):
        self.params = {
            'description': 'Random Forrest with very basic features',
        }

    def fit(self, train_x, train_y):
        train_x = [x.sentence for x in train_x]

        features = self.transform_features(train_x)

        self.model = RandomForestClassifier(n_estimators = 100, random_state = 42)
        self.model.fit(features, train_y)

    def predict(self, test_x):
        test_x = [x.sentence for x in test_x]

        features = self.transform_features(test_x)

        return self.model.predict(features)

    def transform_features(self, data_x):
        features = np.concatenate((
            transformers.count_adjectives(data_x),
            transformers.count_adverbs(data_x),
            transformers.count_singular_pronouns(data_x),
            transformers.count_plural_pronouns(data_x),
            transformers.questions(data_x),
            transformers.exclamations(data_x),
            transformers.periods(data_x),
        ), axis = 1)
        return features
