import numpy as np
import scipy
from sklearn.linear_model import LogisticRegression, SGDClassifier
from sklearn.svm import SVC
from sklearn.ensemble import RandomForestClassifier, AdaBoostClassifier, VotingClassifier
from sklearn.neural_network import MLPClassifier
from sklearn.preprocessing import StandardScaler, MinMaxScaler
from sklearn.tree import DecisionTreeClassifier
from sklearn.metrics import f1_score
from sklearn.feature_extraction.text import TfidfVectorizer

from models.base_estimator import BaseEstimator
from transformers import readability
from transformers import compute_sentiment
from transformers import EmotionsExtractor, merge_pos_counts, subjectivity, polarity
from utils import flatten, get_n_jobs

from models import LogRegLexical
from models import LogRegEmotions
from models import LogRegBaseline
from models import LogRegSubjectivity
from models import LogRegSentiment
from models import LogRegReadability
from models import LogRegLoadedLanguage
from models import LogRegConfusingWords
from models import LogRegProperNouns

TASK_2_POSITIVE_LABEL = 'propaganda'

class StackingVoting(BaseEstimator):
    def __init__(self):
        self.params = {
            'model_description': 'Stacking with hard voting',
            'voting_clf_params': {
                'n_jobs': get_n_jobs(),
                'voting': 'hard',
            }
        }

    def fit(self, train_x, train_y):
        self.clf = VotingClassifier(estimators=[
            ('tfidf', LogRegBaseline()),
            ('confusing_words', LogRegConfusingWords()),
            ('emotions', LogRegEmotions()),
            ('lexical', LogRegLexical()),
            ('loaded_language', LogRegLoadedLanguage()),
            ('readability', LogRegReadability()),
            ('sentiment', LogRegSentiment()),
            ('pronouns', LogRegProperNouns()),
            ('subjectivity', LogRegSubjectivity()),
            ], **self.params['voting_clf_params'])

        self.clf.fit(train_x, train_y)

    def predict(self, test_x):
        return self.clf.predict(test_x)
