import numpy as np
import transformers
from sklearn.svm import SVC
from models.base_estimator import BaseEstimator
from sklearn.feature_extraction.text import TfidfVectorizer

class SVMtfidf(BaseEstimator):
    def __init__(self):
        self.params = {
            'description': 'SVM with tfidf',
            'svm_params': {
                'C': 1,
                'class_weight': 'balanced',
                'kernel': 'rbf',
                'tol': 0.001,
                'verbose': 1,
                'max_iter': -1,
                'random_state': 42,
                'probability': True
            },
        }

    def fit(self, train_x, train_y):
        train_x = [x.sentence for x in train_x]

        self.vectorizer = TfidfVectorizer(sublinear_tf=True)

        tfidf_features = self.vectorizer.fit_transform(train_x)

        self.model = SVC(**self.params['svm_params'])
        self.model.fit(tfidf_features, train_y)

    def predict(self, test_x):
        test_x = [x.sentence for x in test_x]

        tfidf_features = self.vectorizer.transform(test_x)

        return self.model.predict(tfidf_features)
