from sklearn.linear_model import LogisticRegression
import numpy as np
from models.base_estimator import BaseEstimator
from utils import flatten, get_n_jobs
from transformers import has_singular_pronoun

class TestModel(BaseEstimator):
    def __init__(self):
        self.params = {
            'description': 'Logistic Regression with TFIDF (Baseline)',
            'logreg_clf_params': {
                'penalty': 'l2',
                'dual': False,
                'tol': 0.0001,
                'C': 1.0,
                'fit_intercept': True,
                'intercept_scaling': 1,
                'class_weight': None,
                'random_state': None,
                'solver': 'liblinear',
                'max_iter': 100,
                'multi_class': 'ovr',
                'verbose': 0,
                'warm_start': False,
                'n_jobs': 1,
            }
        }

    def fit(self, train_x, train_y):
        train_x = [x.sentence for x in train_x]
        tokens = [sentence.split() for sentence in train_x]

        features = [has_singular_pronoun(x) for x in train_x]
        features = np.reshape(features, (-1, 1))
        self.model = LogisticRegression(**self.params['logreg_clf_params'])

        # self.model = word2vec.Word2Vec(tokenized_sentences, min_count=1)

        self.model.fit(features, train_y)

    def predict(self, test_x):
        test_x = [x.sentence for x in test_x]

        features = [has_singular_pronoun(x) for x in test_x]
        features = np.reshape(features, (-1, 1))
        return self.model.predict(features)

    def get_grid_params(self):
        return {
            'max_iter': (5, 100),
            'solver': ('liblinear', 'sag'),
        }
