import gensim
import numpy as np

from sklearn.linear_model import LogisticRegression
from sklearn.feature_extraction.text import TfidfVectorizer
from models.base_estimator import BaseEstimator
from nltk.corpus import stopwords
from nltk.tokenize import RegexpTokenizer

from utils import get_n_jobs

PRETRAINED_MODEL_PATH = '/Users/dimitar/Downloads/GoogleNews-vectors-negative300.bin'

class Word2Vec(BaseEstimator):
    def __init__(self):
        self.tokenizer = RegexpTokenizer(r'\w+')
        self.stop_words = set(stopwords.words('english'))
        self.word2vec_model = gensim.models.KeyedVectors.load_word2vec_format(PRETRAINED_MODEL_PATH, binary=True)
        self.params = {
            'description': 'Logistic Regression with Word2Vec (Baseline)',
            'logreg_clf_params': {
                'penalty': 'l2',
                'class_weight': 'balanced',
                'solver': 'lbfgs',
                'max_iter': 3000,
                'n_jobs': get_n_jobs(),
            }
        }

    def fit(self, train_x, train_y):
        train_x = [x.sentence for x in train_x]
        train_x = [self.clean_stop_words(x) for x in train_x]
        print('TRAIN: clean stop words')
        # train_x = [np.average(self.word2vec_model.wv[x], 0) for x in train_x]
        # train_x = [np.average(self.word) for x in train_x]
        train_x = [
            self.avg(tokenized_sentence)
            for tokenized_sentence in train_x
        ]
        print('TRAIN: avereage vectors')
        features = train_x

        self.model = LogisticRegression(**self.params['logreg_clf_params'])
        self.model.fit(features, train_y)

    def avg(self, tokenized_sentence):
        a = [self.word2vec_model.wv[word] for word in tokenized_sentence if word in self.word2vec_model.wv]
        if (len(a) == 0):
            return [0 for x in range(0, 300)]
        else:
            return np.average(a, 0)

    def predict(self, test_x):
        test_x = [x.sentence for x in test_x]
        test_x = [self.clean_stop_words(x) for x in test_x]
        print('TRAIN: clean stop words')
        test_x = [
            self.avg(tokenized_sentence)
            for tokenized_sentence in test_x
        ]

        return self.model.predict(test_x)

    def get_grid_params(self):
        return {
            'max_iter': (5, 100),
            'solver': ('liblinear', 'sag'),
        }

    def clean_stop_words(self, text):
        word_tokens = self.tokenizer.tokenize(text)
        return [w for w in word_tokens if not w in self.stop_words]
