from nltk import RegexpTokenizer
from nltk.corpus import stopwords
import gensim
from keras.models import Sequential
from keras.layers import Dense
import numpy as np
from keras.models import Sequential
from keras.layers import Dense
from models import BaseEstimator

np.random.seed(420)

PRETRAINED_MODEL_PATH = '/Users/givanov/Downloads/GoogleNews-vectors-negative300.bin'

class Word2VecNet(BaseEstimator):
    def __init__(self):
        self.tokenizer = RegexpTokenizer(r'\w+')
        self.stop_words = set(stopwords.words('english'))
        self.word2vec_model = gensim.models.KeyedVectors.load_word2vec_format(PRETRAINED_MODEL_PATH, binary=True)
        self.params = {
            'description': 'Keras Neural Net with Word2Vec',
        }

    def fit(self, train_x, train_y):
        train_x = [x.sentence for x in train_x]
        train_x = [self.clean_stop_words(x, self.stop_words) for x in train_x]

        train_x = [
            self.avg(tokenized_sentence)
            for tokenized_sentence in train_x
        ]
        features = np.asarray(train_x)
        
        self.model = Sequential()
        self.model.add(Dense(12, input_dim=300, activation='relu'))
        self.model.add(Dense(8, activation='relu'))
        self.model.add(Dense(1, activation='sigmoid'))
        self.model.compile(loss='binary_crossentropy', optimizer='adam', metrics=['accuracy'])
        self.model.fit(features, train_y, epochs=150, batch_size=10)
    
    def avg(self, tokenized_sentence):
        a = [self.word2vec_model.wv[word] for word in tokenized_sentence if word in self.word2vec_model.wv]
        if (len(a) == 0):
            return [0 for x in range(0, 300)]
        else:
            return np.average(a, 0)

    def predict(self, test_x):
        test_x = [x.sentence for x in test_x]
        test_x = [self.clean_stop_words(x, self.stop_words) for x in test_x]
        print('TRAIN: clean stop words')
        test_x = [
            self.avg(tokenized_sentence)
            for tokenized_sentence in test_x
        ]
        test_x = np.asarray(test_x)
        return self.model.predict(test_x)

    def get_grid_params(self):
        return {
            'max_iter': (5, 100),
            'solver': ('liblinear', 'sag'),
        }

    def clean_stop_words(self, text, stop_words):
        word_tokens = self.tokenizer.tokenize(text)
        filtered_sentence = [w for w in word_tokens if not w in stop_words] 
        filtered_sentence = [] 

        for w in word_tokens: 
            if w not in stop_words: 
                filtered_sentence.append(w)

        return filtered_sentence