from nltk import pos_tag, word_tokenize
import numpy as np

def count_tags_sentence(sentence, tag_prefix):
    """
    Counts the number of words in the given sentence
    whose PoS tag matches the given tag_prefix.

    :param string sentence: the sentence to process
    :param string tag_prefix: Penn Treebank tag prefix (e.g. JJ, RB)
    :return: number of words with the tag, matching the tag_prefix
    """
    tags = pos_tag(word_tokenize(sentence.lower()))
    return len(list(filter(lambda tag: tag[1].startswith(tag_prefix), tags)))

def count_adjectives(sentences):
    return list(map(lambda s: [count_tags_sentence(s, tag_prefix='JJ')], sentences))

def count_adverbs(sentences):
    return list(map(lambda s: [count_tags_sentence(s, tag_prefix='RB')], sentences))

def count_singular_pronouns(sentences):
    return list(map(lambda s: [count_tags_sentence(s, tag_prefix='NNP')], sentences))

def count_plural_pronouns(sentences):
    return list(map(lambda s: [count_tags_sentence(s, tag_prefix='NNPS')], sentences))

def merge_pos_counts(sentences):
    return np.concatenate((
        count_adjectives(sentences),
        count_adverbs(sentences),
        count_singular_pronouns(sentences),
        count_plural_pronouns(sentences)
    ), axis=1)
