import numpy as np
from nltk.tag import pos_tag
from nltk.corpus import sentiwordnet as swn, wordnet

def compute_sentiment(data):
    return [[__compute_score(sent)] for sent in data]

def __compute_score(sentence):
    sent_score = []
    words = pos_tag(sentence.split())
    for t in words:
        word = t[0]
        tag = t[1]
        new_tag = ''
        if tag.startswith('NN'):
            new_tag = wordnet.NOUN
        elif tag.startswith('J'):
            new_tag = wordnet.ADJ
        elif tag.startswith('V'):
            new_tag = wordnet.VERB
        elif tag.startswith('R'):
            new_tag = wordnet.ADV

        if new_tag != '':
            synsets = list(swn.senti_synsets(word, new_tag))
            score = 0.0
            if len(synsets) > 0:
                for syn in synsets:
                    score += syn.pos_score() - syn.neg_score()
                sent_score.append(score / len(synsets))

    if len(sent_score)==0:
        return float(0.0)
    else:
        return np.mean(sent_score)
